=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-10-16 01:25:46 +0000
+++ bzrlib/osutils.py	2006-10-21 02:33:37 +0000
@@ -1090,3 +1090,23 @@
     if _cached_user_encoding is None:
         _cached_user_encoding = 'ascii'
     return _cached_user_encoding
+
+
+def recv_all(socket, bytes):
+    """Receive an exact number of bytes.
+
+    Regular Socket.recv() may return less than the requested number of bytes,
+    dependning on what's in the OS buffer.  MSG_WAITALL is not available
+    on all platforms, but this should work everywhere.  This will return
+    less than the requested amount if the remote end closes.
+
+    This isn't optimized and is intended mostly for use in testing.
+    """
+    b = ''
+    while len(b) < bytes:
+        new = socket.recv(bytes - len(b))
+        if new == '':
+            break # eof
+        b += new
+    return b
+

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-10-16 01:50:48 +0000
+++ bzrlib/tests/test_http.py	2006-10-21 02:33:37 +0000
@@ -26,6 +26,7 @@
 
 import bzrlib
 from bzrlib.errors import DependencyNotPresent, UnsupportedProtocol
+from bzrlib import osutils
 from bzrlib.tests import TestCase, TestSkipped
 from bzrlib.transport import get_transport, Transport
 from bzrlib.transport.http import extract_auth, HttpTransportBase
@@ -340,5 +341,5 @@
         sock.connect((server.host, server.port))
         sock.sendall('abc')
         self.assertEqual('HTTP/1.1 200 OK\r\n',
-                         sock.recv(4096, socket.MSG_WAITALL))
+                         osutils.recv_all(sock, 4096))
         self.assertEqual('abc', server.received_bytes)

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2006-10-09 07:34:15 +0000
+++ bzrlib/tests/test_smart_transport.py	2006-10-21 02:33:37 +0000
@@ -97,7 +97,7 @@
         """
         def _receive_bytes_on_server():
             connection, address = sock.accept()
-            bytes.append(connection.recv(3, socket.MSG_WAITALL))
+            bytes.append(osutils.recv_all(connection, 3))
             connection.close()
         t = threading.Thread(target=_receive_bytes_on_server)
         t.start()

